from typing import AsyncIterable

from ptvp35 import Db, KVJson
from v6d2ctx.context import Explicit

from v6d3music.utils.argctx import ArgCtx, InfoCtx
from v6d3music.config import myroot
from v6d3music.utils.presets import allowed_effects


__all__ = ('effects_db', 'default_effects', 'set_default_effects', 'entries_effects_for_args',)


effects_db = Db(myroot / 'effects.db', kvfactory=KVJson())


def default_effects(gid: int) -> str | None:
    effects = effects_db.get(gid, None)
    if effects in allowed_effects:
        return effects
    else:
        return None


async def set_default_effects(gid: int, effects: str | None) -> None:
    if effects is not None and effects not in allowed_effects:
        raise Explicit('these effects are not allowed')
    await effects_db.set(gid, effects)


async def entries_effects_for_args(args: list[str], gid: int) -> AsyncIterable[InfoCtx]:
    for ctx in ArgCtx(default_effects(gid), args).sources:
        async for it in ctx.entries():
            yield it
